//************************************************************************//
//									  //
//  Copyright 2013 Bertram Kopf (bertram@ep1.rub.de)			  //
//  	      	   Julian Pychy (julian@ep1.rub.de)			  //
//          	   - Ruhr-Universität Bochum 				  //
//									  //
//  This file is part of Pawian.					  //
//									  //
//  Pawian is free software: you can redistribute it and/or modify	  //
//  it under the terms of the GNU General Public License as published by  //
//  the Free Software Foundation, either version 3 of the License, or 	  //
//  (at your option) any later version.	 	      	  	   	  //
//									  //
//  Pawian is distributed in the hope that it will be useful,		  //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of	  //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	  //
//  GNU General Public License for more details.	      		  //
//									  //
//  You should have received a copy of the GNU General Public License     //
//  along with Pawian.  If not, see <http://www.gnu.org/licenses/>.	  //
//									  //
//************************************************************************//

// AbsEnv class definition file. -*- C++ -*-
// Copyright 2013 Bertram Kopf

#pragma once

#include <iostream>
#include <vector>
#include <map>
#include <vector>
#include <string>
#include <sstream>
#include <memory>

#include "PwaUtils/DataUtils.hh"
#include "PwaUtils/AbsHist.hh"
#include "PwaUtils/WaveContribution.hh"

class Particle;
class ParticleTable;
class AbsDecayList;
class IsobarDecayList;
class ParserBase;

class AbsEnv {

public:
  ~AbsEnv();
  
  void setup(ParserBase* theParser);

  ParticleTable* particleTable() {return _particleTable;}

  const int noFinalStateParticles() {return _noFinalStateParticles;}
  std::vector<Particle*> finalStateParticles() {return _finalStateParticles;}
  std::vector<std::pair<Particle*, Particle*> > producedParticlePairs() {return _producedParticlePairs;}
  std::shared_ptr<AbsDecayList> absDecayList() {return _absDecList;}
  std::shared_ptr<AbsDecayList> prodDecayList() {return _prodDecList;}
  const std::string outputFileNameSuffix() const {return _outputFileNameSuffix;}
  const std::string serializationFileName() const {return _serializationFileName;}
  std::vector<std::vector<std::string> >& histMassSystems() {return _histMassSystems;} 
  std::vector<std::shared_ptr<angleHistData> >& angleHistDataVec() {return _angleHistDataVec;}
  std::vector<std::shared_ptr<angleHistData2D> >& angleHistDataVec2D() {return _angleHistDataVec2D;}
  std::vector<std::shared_ptr<calcContributionData> >& calcContributionDataVec() {return _calcContributionDataVec;}
  bool useMassRange() {return _useMassRange;}
  double massRangeMin() {return _massMin;}
  double massRangeMax() {return _massMax;}
  std::vector<unsigned int>& particleIndicesMassRange() {return _particleIndicesMassRange;}
  Vector4<double>&  initial4Vec() {return _initial4Vec;} 
  ParserBase* parser() {return _theParser;}

protected:  

  AbsEnv();
  bool _alreadySetUp;
  int _noFinalStateParticles;
  ParticleTable* _particleTable;
  std::vector<Particle*> _finalStateParticles;
  std::vector<std::pair<Particle*, Particle*> > _producedParticlePairs;
  std::shared_ptr<AbsDecayList> _absDecList;
  std::shared_ptr<AbsDecayList> _prodDecList;
  std::string _outputFileNameSuffix;
  std::string _serializationFileName;
  std::vector<std::vector<std::string> > _histMassSystems;
  std::vector<std::shared_ptr<angleHistData> > _angleHistDataVec;
  std::vector<std::shared_ptr<angleHistData2D> > _angleHistDataVec2D;
  std::vector<std::shared_ptr<calcContributionData> > _calcContributionDataVec;
  bool _useMassRange;
  double _massMin;
  double _massMax;
  std::vector<unsigned int> _particleIndicesMassRange;
  Vector4<double> _initial4Vec;
  std::map<std::string, double> _preFactorMap;
  ParserBase* _theParser;
};
